/*
 *  Memory for Uzebox Version 1.1
 *  Copyright (C) 2011  Hartmut Wendt
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdbool.h>
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include "kernel/uzebox.h"
#include "data/backgroundTiles.pic.inc"
#include "data/fonts.pic.inc"
#include "data/patches.inc"
#include "data/memory-tracks.inc"



/* global definitons */
// program modes
#define PM_Intro	0	// program mode intro
#define PM_1PLAYER	1	// program mode 1 player mode
#define PM_2PLAYER  2	// program mode 2 player mode
#define PM_GAMEMENU 3	// program mode game menu
#define PM_HoF_view 20	// program mode view hall of fame
#define PM_HoF_edit 21	// program mode edit hall of fame


// 8-bit, 255 period LFSR (for generating pseudo-random numbers)
#define PRNG_NEXT() (prng = ((u8)((prng>>1) | ((prng^(prng>>2)^(prng>>3)^(prng>>4))<<7))))
#define max_cards 48


#define MAX(x,y) ((x)>(y) ? (x) : (y))


struct EepromBlockStruct ebs;


u8 playing_field[max_cards];
int menu_background[32];

u8 prng; // Pseudo-random number generator
int CurBackTile;	// background tile under cursor
u8 CurPosX;	// background tile under cursor
u8 CurPosY;	// background tile under cursor
u8 PosX=0, PosY=0;
u8 AniCnt;  // Animation counter
int iSCORE;			// Highscore
u8 program_mode;	// program mode (intro, 1 player mode, 2 player mode, ....
u8 PLAYER;	// 0 = Player 1 / 1 = Player 2
u8 ani_count=0;		// counter for animation
u8 Music_on = true;



/*** function prototypes *****************************************************************/
void init(void);
void clear_playing_field(void);
void create_new_playing_field(void);
void show_playing_field(u8 visible);
const int * get_card_map(u8 card);
int gettile(u8 xx, u8 yy);
u8 getcard(u8 xx, u8 yy);
void flipcard(u8 card, u8 back);
void hidecards(u8 *card1, u8 *card2);
u8 check_finish(void);
void set_cursor(u8 xx, u8 yy);
void set_PM_mode(u8 mode);
void hide_cursor(void); 
void show_cursor(u8 xx, u8 yy);
void msg_window(u8 x1, u8 y1, u8 x2, u8 y2);
void save_background(u8 x1, u8 y1,u8 x2, u8 y2);
void restore_background(u8 x1, u8 y1,u8 x2, u8 y2); 
u8 check_highscore(void);
void copy_highsore(u8 entry_from, u8 entry_to);
void clear_highsore(u8 entry);
u8 set_def_highscore(void);
u8 view_highscore_entry(u8 x, u8 y, u8 entry, u8 load_data);
void edit_highscore_entry(u8 entry, u8 cursor_pos, u8 b_mode);
void show_highscore_char(u8 entry, u8 position, u8 cursor_on);
void fade_out_volume(void);


void init(void)
// init program
{
  // init font table
  SetFontTable(fonts);
  // init music	
  InitMusicPlayer(patches);
  // load into screen
  set_PM_mode(PM_Intro);
  // init random generator
  prng = 15;
  //Use block 10
  ebs.id = 10;
  if (!isEepromFormatted())
     return;

  if (EEPROM_ERROR_BLOCK_NOT_FOUND == EepromReadBlock(10,&ebs))
  {
	set_def_highscore();
  }	
}



int main(){
int ibuttons=0,ibuttons_old;
u8 card1=99,card2=99;
u8 PairsPlayer1 = 0, PairsPlayer2 = 0;
u8 old_mode = 0;
u8 b = 0;
  // init program
  init();        
  // proceed game	
  while(1)
  {
    WaitVsync(3);	  
    // get controller state
    ibuttons_old = ibuttons;
	ibuttons = ReadJoypad(PLAYER);
    switch(program_mode)
	{
	  // proceed intro mode	
	  case PM_Intro:
		// 
     	if ((BTN_A & ibuttons) && !(ibuttons_old & BTN_A)) {
		  // wait for release of button A 
		  while (BTN_A & ReadJoypad(PLAYER)) {
		    prng++; 		
			WaitVsync(1);
		  }
      	  prng = MAX(prng,1);
		  if (PosY == 0) set_PM_mode(PM_1PLAYER);	
		  else if (PosY == 1) {
		    PairsPlayer1 = 0;
			PairsPlayer2 = 0;
		    set_PM_mode(PM_2PLAYER);
  		  } else if (PosY == 2) {
		    iSCORE = 9999; 
		    set_PM_mode(PM_HoF_view);
		  } else {
		    if (Music_on) {
			  Music_on = false;
			  Print(17,15,PSTR("MUSIC OFF "));
		    } else {
			  Music_on = true;			  
			  Print(17,15,PSTR("MUSIC ON "));
			}
		  }
 		}
		// UP button
		if ((ibuttons & BTN_UP) && !(ibuttons_old & BTN_UP)) {
		  SetTile(15,12+PosY,0); 	
		  if (PosY) PosY--;	
		  SetTile(15,12+PosY,124); 	

		// DOWN button
		} else if ((ibuttons & BTN_DOWN) && !(ibuttons_old & BTN_DOWN)) {
		  SetTile(15,12+PosY,0); 	
		  if (PosY<3) PosY++;	
		  SetTile(15,12+PosY,124); 	
		}
				
        break;
		


	  // proceed 1 player mode
  	  case PM_1PLAYER:
		// check controller action
     	if ((BTN_LEFT & ibuttons) && (PosX>5)) PosX -= 5;
     	else if ((BTN_RIGHT & ibuttons) && (PosX<34)) PosX += 5;
     	if ((BTN_UP & ibuttons) && (PosY>5)) PosY -= 4;
     	else if ((BTN_DOWN & ibuttons) && (PosY<20)) PosY += 4;		
		set_cursor(PosX,PosY);

		if (BTN_B & ibuttons) {
		  old_mode = program_mode;		
		  set_PM_mode(PM_GAMEMENU);	
		  break;	
		}
		
		// flip card
		if (BTN_A & ibuttons) {
		   
   		  if (card1 == 99) {
		  	// show card 1
			TriggerFx(4, 70, false);
		    card1 = getcard(PosX,PosY);				    

			if (!(playing_field[card1])) {
			  card1 = 99; 
			  continue;
			}
			flipcard(card1,false);
			WaitVsync(1);
			show_cursor(PosX,PosY);
		    

		  } else {
		    // show card 2
		    card2 = getcard(PosX,PosY);    		

			if (!(playing_field[card2]) || (card1 == card2)) continue;
			flipcard(card2,false);
			WaitVsync(1);			
			iSCORE++;
			PrintInt(38,26,iSCORE,true);
			// check cards
			if (playing_field[card1] == playing_field[card2]) {
			  // cards equal 
			  TriggerFx(5, 70, false);
			  hidecards(&card1,&card2);
			  if (check_finish()) {		
			    b = 0;	    
				set_PM_mode(PM_HoF_view);
				break;
			  }

			} else {
			  // cards not equal 
			  TriggerFx(6, 70, false);	
			  // show cards
			  while (BTN_A & ReadJoypad(PLAYER)); //wait for release of button A	
			  for (b=0; b<90; b++) {
		    	if (BTN_A & ReadJoypad(PLAYER)) break; 
  			    WaitVsync(1);
			  }
			  while (BTN_A & ReadJoypad(PLAYER)); //wait for release of button A
   			  flipcard(card1,true);
			  flipcard(card2,true);
			  card1 = 99;
			  card2 = 99;
			}
			PosX = 17;
			PosY = 15;
 		    show_cursor(PosX,PosY);			  

		  }
		}
	    break;	

	  // proceed 2 player mode
  	  case PM_2PLAYER:
		// animate active player
		if (ani_count & 0x04) {
		  // Player 2 
		  if (PLAYER) Print(26,26,PSTR("PLAYER 2:"));	
		  // Player 1 
		  else Print(1,26,PSTR("PLAYER 1:"));
		} else {
		  if (PLAYER) Print(26,26,PSTR("         "));	
		  // Player 1 
		  else Print(1,26,PSTR("         "));
		}		
		// check controller action
		if ((BTN_LEFT & ibuttons) && (PosX>5)) PosX -= 5;
     	else if ((BTN_RIGHT & ibuttons) && (PosX<34)) PosX += 5;
     	if ((BTN_UP & ibuttons) && (PosY>5)) PosY -= 4;
     	else if ((BTN_DOWN & ibuttons) && (PosY<20)) PosY += 4;	
		set_cursor(PosX,PosY);
		
		if (BTN_B & ibuttons) {
		  old_mode = program_mode;		
		  set_PM_mode(PM_GAMEMENU);	
		  break;	
		}

		// flip card
		if (BTN_A & ibuttons) {
		   
   		  if (card1 == 99) {
		  	// show card 1
			TriggerFx(4, 70, false);
		    card1 = getcard(PosX,PosY);					    

			if (!(playing_field[card1])) {
			  card1 = 99; 
			  continue;
			}
			flipcard(card1,false);
			WaitVsync(1);
			show_cursor(PosX,PosY);
		    

		  } else {
		    // show card 2
		    card2 = getcard(PosX,PosY);    		

			if (!(playing_field[card2]) || (card1 == card2)) continue;
			flipcard(card2,false);
			WaitVsync(1);			
			// check cards
  		    if (PLAYER) Print(26,26,PSTR("PLAYER 2:"));
		    else Print(1,26,PSTR("PLAYER 1:"));

			if (playing_field[card1] == playing_field[card2]) {
			  // cards equal 
			  TriggerFx(5, 70, false);
			  hidecards(&card1,&card2);
  			  // show new score 
			  if (PLAYER) {
			    // for player 2
			    PairsPlayer2++;
				PrintByte(38,26,PairsPlayer2,true);
		      } else {
			    // for player 1
			    PairsPlayer1++;
				PrintByte(13,26,PairsPlayer1,true);
			  }
			  // all pairs found	
			  if (check_finish()) {
                msg_window(10,10,29,19);
				if (PairsPlayer1 > PairsPlayer2) Print(17,13,PSTR("PLAYER 1"));
				else if (PairsPlayer2 > PairsPlayer1) Print(17,13,PSTR("PLAYER 2"));
				else Print(17,13,PSTR("NOBODY"));
				Print(12,16,PSTR("WINS THIS MATCH!"));
				WaitVsync(60);
			    PosY = 0;
				set_PM_mode(PM_Intro);
				break;
			  }

			} else {
			  // cards not equal 
			  TriggerFx(6, 70, false);	
			  // show cards
			  while (BTN_A & ReadJoypad(PLAYER)); //wait for release of button A	
			  for (b=0; b<90; b++) {
		    	if (BTN_A & ReadJoypad(PLAYER)) break; 
  			    WaitVsync(1);
			  }
			  while (BTN_A & ReadJoypad(PLAYER)); //wait for release of button A
   			  flipcard(card1,true);
			  flipcard(card2,true);
			  card1 = 99;
			  card2 = 99;
			  // switch player		  
  		      if (PLAYER) PLAYER = 0;
			  else PLAYER = 1;
			}
			// set cursor to center position
			PosX = 17;
			PosY = 15;
 		    show_cursor(PosX,PosY);				

		  }

		}
	    break;	

	  // menu in games
  	  case PM_GAMEMENU:
		// UP button
		if ((ibuttons & BTN_UP) && !(ibuttons_old & BTN_UP)) {
		  SetTile(17,13+PosY,401); 	
		  if (PosY) PosY--;	
		  SetTile(17,13+PosY,412); 	

		// DOWN button
		} else if ((ibuttons & BTN_DOWN) && !(ibuttons_old & BTN_DOWN)) {
		  SetTile(17,13+PosY,401); 	
		  if (PosY<1) PosY++;	
		  SetTile(17,13+PosY,412); 	
		}
		if ((BTN_A & ibuttons) && !(ibuttons_old & BTN_A)) {
		  // wait for release of button A 
		  while (BTN_A & ReadJoypad(PLAYER));
		  ibuttons = 0;
		  if (PosY) {			
			PosY = 0;
			if (Music_on) fade_out_volume();
			card1 = 99;
			card2 = 99;
			set_PM_mode(PM_Intro);
		  } else {
 	  		program_mode = old_mode;		
			restore_background(16,12,23,15);
			PosX = 19;
			PosY = 13;
			show_cursor(PosX,PosY);
		  }
		}
	    break;

	  case PM_HoF_view:				
		// time control				
		if (b <= 200) b++;
		if (b == 100) Print(14,24,PSTR("PRESS BUTTON A"));
		if (((ibuttons & BTN_A) && !(ibuttons_old & BTN_A) && (b > 50)) || (b > 200))
		{
		  if (Music_on) fade_out_volume();
		  b = 0;	
		  set_PM_mode(PM_Intro);
		}
		break;

	  case PM_HoF_edit:				
		// cursor blinking
		b++;
		if (b >= 10) b = 0;
		// proceed cursor position with left & right button
		if ((ibuttons & BTN_RIGHT) && !(ibuttons_old & BTN_RIGHT)) {
		  if (PosX < 7) {       	   
		  	show_highscore_char(PosY - 1, PosX, 0); 		 
		    PosX++; 			
          }
		}
		if ((ibuttons & BTN_LEFT) && !(ibuttons_old & BTN_LEFT)) {		 
 		  if (PosX) {
		    show_highscore_char(PosY - 1, PosX, 0); 
		    PosX--; 
          }
		}
		// chose character up & down button
		if ((ibuttons & BTN_UP) && !(ibuttons_old & BTN_UP)) {
		  edit_highscore_entry(PosY,PosX,BTN_UP); 
		}
		else if ((ibuttons & BTN_DOWN) && !(ibuttons_old & BTN_DOWN)) {		 
 		  edit_highscore_entry(PosY,PosX,BTN_DOWN); 
		}     
		// show cursor
		show_highscore_char(PosY - 1, PosX, b > 4);

		// store new entry
		if (ibuttons & BTN_A)   
		{
		  // store new highscore 
		  EepromWriteBlock(&ebs);
		  if (Music_on) fade_out_volume();
		  set_PM_mode(PM_Intro);
		}
		break;



    }	

	ani_count++;	// increase counter for animation
  }
  

} 


void set_PM_mode(u8 mode) {
// set parameters, tiles, background etc for choosed program mode
		
	switch (mode)
	{
	  
	  case	PM_Intro:
   		PosY = 0;	
   	    PLAYER = 0;    
	    // cursor is invisible now
	  	hide_cursor();	    
	    StopSong();
		ClearVram();
		//Set the tile table to use.
		SetTileTable(LogoTiles);
		WaitVsync(2);
		// draw memory logo
  		 DrawMap(1,0,logo_map);
     	Print(15,8,PSTR("VERSION 1.1"));

		Print(17,12,PSTR("1 PLAYER"));
		Print(17,13,PSTR("2 PLAYERS"));
		Print(17,14,PSTR("HALL OF FAME"));
		if (Music_on) Print(17,15,PSTR("MUSIC ON "));
		else Print(17,15,PSTR("MUSIC OFF"));

  	    // cursor
		SetTile(15,12,124); 	


     	Print(9,19,PSTR("(C) 2011 HARTMUT WENDT"));
		Print(7,21,PSTR("SOUNDTRACK BY CARSTEN KUNZ"));
		
		Print(7,24,PSTR("LICENCED UNDER GNU GPL V3"));
		Print(10,26,PSTR("WWW.HWHARDSOFT.DE.VU"));
		WaitVsync(10);
		SetMasterVolume(130);	
		break;

      case	PM_HoF_view:
	    // switch music on
	    SetMasterVolume(130);
		if (Music_on) StartSong(champions_song);
	    PosY = check_highscore();
	    if (PosY == 2) copy_highsore(1,2);
	    if (PosY == 1) {
		  copy_highsore(1,2);
		  copy_highsore(0,1);
        }
		clear_highsore(PosY - 1);
		// reset cursor to left position
		PosX = 0;
	  	// Set the font and tiles to use.
  		SetTileTable(uMTiles);
	    ClearVram();
		msg_window(12,8,29,19);	
		Print(15,10,PSTR("HALL OF FAME"));
        view_highscore_entry(14,13,1,!(PosY));
        view_highscore_entry(14,15,2,!(PosY));
        view_highscore_entry(14,17,3,!(PosY));  
		if (PosY)  {
		  mode = PM_HoF_edit;
		  Print(14,3,PSTR("CONGRATULATION"));	
		  Print(14,5,PSTR("NEW HIGHSCORE!"));	
		  Print(13,22,PSTR("ENTER YOUR NAME"));	
		  Print(12,24,PSTR("AND PRESS BUTTON A"));	
		}
		break;	  

	  case	PM_1PLAYER:	    
	    PLAYER = 0;    
		iSCORE = 0;		
		PosX = 17;	
		PosY = 15;	
    	//Set the tile table to use.
		SetTileTable(uMTiles);
        // clear screen
	    ClearVram();
        create_new_playing_field();
	    show_playing_field(false);	  
	    Print(1,26,PSTR("1 PLAYER MODE"));	
	    Print(27,26,PSTR("MOVES:"));	
	    PrintInt(38,26,iSCORE,true);
        
		// cursor is visible now
	  	show_cursor(1,1);	    	  	

		//start sound track
		if (Music_on) StartSong(memory_song);		
		break;

	  case	PM_2PLAYER:	
	    PLAYER = 0;    
		PosX = 17;	
		PosY = 15;	
    	//Set the tile table to use.
		SetTileTable(uMTiles);
        // clear screen
	    ClearVram();
        create_new_playing_field();
	    show_playing_field(false);	  
	    Print(1,26,PSTR("PLAYER 1: 000"));	
	    Print(26,26,PSTR("PLAYER 2: 000"));	
        
		// cursor is visible now
	  	show_cursor(1,1);	    	  	

		//start sound track
		if (Music_on) StartSong(memory_song);		
		break;

	  case PM_GAMEMENU:
	    hide_cursor();
	    // save background under menu window
		save_background(16,12,23,15);
		msg_window(16,12,23,15);
		Print(19,13,PSTR("BACK"));
		Print(19,14,PSTR("MENU"));
  	    // cursor
		SetTile(17,13,412); 	
		PosY = 0;
	    break;

	}
	program_mode = mode;

}


void save_background(u8 x1, u8 y1,u8 x2, u8 y2) {
// save the background in defined area
u8 xi;
  for(;y1 <= y2; y1++) {
    for(xi = x1;xi <= x2; xi++) {
	  menu_background[((y2-y1) * 8) + (x2-xi)] = gettile(xi,y1);
	}
  }
}


void restore_background(u8 x1, u8 y1,u8 x2, u8 y2) {
// restore the background in defined area
u8 xi;
  for(;y1 <= y2; y1++) {
    for(xi = x1;xi <= x2; xi++) {
	  SetTile(xi,y1, menu_background[((y2-y1) * 8) + (x2-xi)]);	   
	}
  }
}



void clear_playing_field(void) {
// clear the playing field
  u8 u1;
  for(u1 = 0; u1 < max_cards; u1++) playing_field[u1] = 0;
}


void create_new_playing_field(void) {
// creates a new randomized playing field
  u8 u1,u2;

  clear_playing_field();

  for(u1 = 1; u1 <= (max_cards / 2); u1++) {
  
    // card 1 of 2
	
	//PrintInt(10,25,prng,true);
	do 
	{	  
	  u2 = PRNG_NEXT() % 48;
     //PrintInt(20,26,u2,true);


	} while(playing_field[u2]);
	// PrintInt(10,26,u1,true);


    playing_field[u2] = u1;
    // card 2 of 2
	do 
	{	  
	  u2 = PRNG_NEXT() % 48;	  
     // PrintInt(20,27,u2,true);	


	} while(playing_field[u2]);
	// PrintInt(10,27,u1,true);


    playing_field[u2] = u1;
  }

}


// shows the new created playing field on the screen
void show_playing_field(u8 visible){
  u8 u1;
  for(u1 = 0; u1 < max_cards; u1++) {
	if (visible) DrawMap((u1 % 8) * 5, (u1 / 8) * 4 + 1,get_card_map(playing_field[u1]));
	else DrawMap((u1 % 8) * 5, (u1 / 8) * 4 + 1,map_card1);
  }

}


// returns the map for a card
const int * get_card_map(u8 card) {
    switch(card) {
	  case 1: 
	    return(map_card2);
	  	break;

	  case 2: 
	    return(map_card3);
	  	break;

	  case 3: 
	    return(map_card4);
	  	break;

	  case 4: 
	    return(map_card5);
	  	break;

	  case 5: 
	    return(map_card6);
	  	break;

	  case 6: 
	    return(map_card7);
	  	break;

	  case 7: 
	    return(map_card8);
	  	break;

	  case 8: 
	    return(map_card9);
	  	break;

	  case 9: 
	    return(map_card10);
	  	break;

	  case 10: 
	    return(map_card11);
	  	break;

	  case 11: 
	    return(map_card12);
	  	break;

	  case 12: 
	    return(map_card13);
	  	break;

	  case 13: 
	    return(map_card14);
	  	break;

	  case 14: 
	    return(map_card15);
	  	break;

	  case 15: 
	    return(map_card16);
	  	break;

	  case 16: 
	    return(map_card17);
	  	break;

	  case 17: 
	    return(map_card18);
	  	break;

	  case 18: 
	    return(map_card19);
	  	break;

	  case 19: 
	    return(map_card20);
	  	break;

	  case 20: 
	    return(map_card21);
	  	break;

	  case 21: 
	    return(map_card22);
	  	break;

	  case 22: 
	    return(map_card23);
	  	break;

	  case 23: 
	    return(map_card24);
	  	break;

	  case 24: 
	    return(map_card25);
	  	break;

      default:
	    return(map_card1);
	}

}


int gettile(u8 xx, u8 yy) {
// get tile from VRAM for cursor functions
int i1 = (yy * 80) + (xx * 2);

  return(((vram[i1 + 1] * 256) + vram[i1] - (int)(uMTiles)) / 48);
}


u8 getcard(u8 xx, u8 yy) {
// get card from playing ground according the cursor position
  return((((yy - 1) / 4) * 8) + (xx / 5));

}


void flipcard(u8 card, u8 back) {
// flip the card
  hide_cursor();
  if (back) DrawMap((card % 8) * 5, (card / 8) * 4 + 1,map_card1);
  else DrawMap((card % 8) * 5, (card / 8) * 4 + 1,get_card_map(playing_field[card]));
}


void hidecards(u8 *card1, u8 *card2) {
// hide a card
u8 u1;
  // wait for button A relase
  while (BTN_A & ReadJoypad(PLAYER));
  hide_cursor();
  // animation
  for(u1 = 0; u1 < 3; u1++) {    
	DrawMap((*card1 % 8) * 5, (*card1 / 8) * 4 + 1,map_card_black);
    DrawMap((*card2 % 8) * 5, (*card2 / 8) * 4 + 1,map_card_black);
	if (BTN_A & ReadJoypad(PLAYER)) break; // break animation
	WaitVsync(10);
	DrawMap((*card1 % 8) * 5, (*card1 / 8) * 4 + 1,get_card_map(playing_field[*card1]));
	DrawMap((*card2 % 8) * 5, (*card2 / 8) * 4 + 1,get_card_map(playing_field[*card2]));
	if (BTN_A & ReadJoypad(PLAYER)) break; // break animation
	WaitVsync(10);
  }
  // hide cards
  DrawMap((*card1 % 8) * 5, (*card1 / 8) * 4 + 1,map_card_black);
  DrawMap((*card2 % 8) * 5, (*card2 / 8) * 4 + 1,map_card_black);
  playing_field[*card1] = 0;
  playing_field[*card2] = 0;
  *card1 = 99;
  *card2 = 99;
  while (BTN_A & ReadJoypad(PLAYER)); //wait for button A release
}


u8 check_finish(void) {
// check the play ground - all pairs found?
u8 u1;
  for(u1 = 0; u1 < 48; u1++) {
	if (playing_field[u1]) return(false);	
  }
  return(true);	
}




void set_cursor(u8 xx, u8 yy) {
// set and animate cursor
u8 a;
  // cursor invisible?
  if (CurBackTile == 0xFFFF) return;
  // restore background at old cursor position 
  if ((CurPosX < 40) & (CurPosY < 28)) SetTile(CurPosX,CurPosY,CurBackTile); 
  // save tile at new position
  CurBackTile = gettile(xx,yy);  
  // save coordinates	
  CurPosX = xx;
  CurPosY = yy;
  // cursor blinking
  a = ani_count % 3;	// counter for animated sprite frame
  SetTile(CurPosX,CurPosY,CURSOR_START + a);
}


void hide_cursor(void) {
// hide cursor
  // restore background at cursor position
  SetTile(CurPosX,CurPosY,CurBackTile);
  // mark cursor as invisible
  CurBackTile = 0xFFFF;
}


void show_cursor(u8 xx, u8 yy) {
// shows cursor
  // mark cursor as visible
  CurBackTile = gettile(xx,yy);
  // save coordinates
  CurPosX = xx;
  CurPosY = yy;

}


void msg_window(u8 x1, u8 y1, u8 x2, u8 y2) {
// draw a window with frame and black backgound on the screen

    // window backgound
	Fill(x1 + 1, y1 + 1, x2 - x1 - 1, y2 - y1 - 1,401);
	// upper frame
	Fill(x1 + 1, y1, x2 - x1 - 1, 1,411);
	// lower frame
	Fill(x1 + 1, y2, x2 - x1 - 1, 1,410);
	// left frame
	Fill(x1 , y1 + 1, 1, y2 - y1 - 1,409);
	// right frame
	Fill(x2, y1 + 1, 1 , y2 - y1 - 1,409);
	// upper, left corner
	SetTile(x1,y1,405);
	// upper, right corner
	SetTile(x2,y1,406);
	// lower, left corner
	SetTile(x1,y2,407);
	// lower, right corner
	SetTile(x2,y2,408);
}	




u8 check_highscore(void) {
// check the actual highsore
u8 a;
int i1;
   // read the eeprom block
  if (!isEepromFormatted() || EepromReadBlock(10, &ebs))
        return(0);   
  for(a=0; a<3; a++) {
    i1 = (ebs.data[(a * 10)+8] * 256) + ebs.data[(a * 10)+9];
    if (iSCORE < i1) return(a + 1);
  }

  // highscore is lower as saved highscores 
  return(0);
}



void copy_highsore(u8 entry_from, u8 entry_to) {
// copy a highscore entry to another slot
u8 a;
   // read the eeprom block
  for(a=0; a<10; a++) {
    ebs.data[(entry_to * 10) + a] = ebs.data[(entry_from * 10) + a];
  } 
}


void clear_highsore(u8 entry) {
// clear the name in actual entry and set the score to highscore
u8 a;
  // clear name 
  for(a=0; a<8; a++) {
    ebs.data[(entry * 10) + a] = 0x20;
  }   
  // set score
  ebs.data[(entry * 10) + 8] = iSCORE / 256;
  ebs.data[(entry * 10) + 9] = iSCORE % 256;
}



u8 set_def_highscore(void) {
// write the default highscore list in the EEPROM
  // entry 1
  ebs.data[0] = 'H';
  ebs.data[1] = 'A';
  ebs.data[2] = 'R';
  ebs.data[3] = 'T';
  ebs.data[4] = 'M';
  ebs.data[5] = 'U';
  ebs.data[6] = 'T';
  ebs.data[7] = ' ';
  ebs.data[8] = 0x00;
  ebs.data[9] = 0x32;
  // entry 2
  ebs.data[10] = 'M';
  ebs.data[11] = 'A';
  ebs.data[12] = 'R';
  ebs.data[13] = 'I';
  ebs.data[14] = 'E';
  ebs.data[15] = ' ';
  ebs.data[16] = ' ';
  ebs.data[17] = ' ';
  ebs.data[18] = 0x00;
  ebs.data[19] = 0x3C;
  // entry 3
  ebs.data[20] = 'L';
  ebs.data[21] = 'U';
  ebs.data[22] = 'K';
  ebs.data[23] = 'A';
  ebs.data[24] = 'S';
  ebs.data[25] = ' ';
  ebs.data[26] = ' ';
  ebs.data[27] = ' ';
  ebs.data[28] = 0x00;
  ebs.data[29] = 0x46;
  return(EepromWriteBlock(&ebs));
}


u8 view_highscore_entry(u8 x, u8 y, u8 entry, u8 load_data) {
// shows an entry of the higscore
u8 a,c;

  // read the eeprom block
  if (load_data)
  {
    if (!isEepromFormatted() || EepromReadBlock(10, &ebs))
        return(1);   
  }
  entry--;
  for(a = 0; a < 8;a++) {
	c = ebs.data[a + (entry * 10)];  
	PrintChar(x + a, y, c);  
  }
  PrintInt(x + 13, y, (ebs.data[(entry * 10)+8] * 256) + ebs.data[(entry * 10)+9], true);
  return(0);
}



void edit_highscore_entry(u8 entry, u8 cursor_pos, u8 b_mode) {
// edit and view and char in the name of choosed entry    
entry--;
u8 c = ebs.data[(entry * 10) + cursor_pos];
  // proceed up & down button
  if (b_mode == BTN_UP) {
     c++;
     if (c > 'Z') c = ' '; 
     else if (c == '!') c = 'A';
  }
  if (b_mode == BTN_DOWN) {		 
     c--;      
     if (c == 0x1F) c = 'Z';
	 else if (c < 'A') c = ' ';
  }
  ebs.data[(entry * 10) + cursor_pos] = c;

}


void show_highscore_char(u8 entry, u8 position, u8 cursor_on) {
// shows a char of edited name
u8 c = ebs.data[(entry * 10) + position];
    if (cursor_on) PrintChar(14 + position, (entry * 2) + 13, '#');   // show '_'
    else if (c == ' ') PrintChar(14 + position, (entry * 2) + 13, ' ');	// space
    else PrintChar(14 + position, (entry * 2) + 13, c); 	
}


void fade_out_volume(void) {
// fade out the master volume
u8 a;
 for(a = 130; a ; a-=2) {
   SetMasterVolume(a);	
   WaitVsync(1);
 } 
 StopSong();
 InitMusicPlayer(patches);
 WaitVsync(3);

}

